@@ -1,7 +1,5 @@
 use erupt::vk;
-use rand::Rng;
-use rand_xoshiro::rand_core::SeedableRng;
-use rand_xoshiro::Xoshiro256PlusPlus;
+use romu::Rng;
 
 use vk_alloc::{Allocation, AllocationDescriptor, Allocator, AllocatorDescriptor, MemoryLocation};
 
@@ -54,7 +52,7 @@ fn allocator_simple_free() {
                         .alignment(512)
                         .size(1024)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -99,7 +97,7 @@ fn allocator_allocation_1024() {
                                 .alignment(512)
                                 .size(1024)
                                 .memory_type_bits(u32::MAX)
-                                .build(),
+                                .build_dangling(),
                             lifetime: TestLifetime::Static,
                             is_dedicated: false,
                             is_optimal: false,
@@ -160,7 +158,7 @@ fn allocator_allocation_256() {
                                 .alignment(1024)
                                 .size(256)
                                 .memory_type_bits(u32::MAX)
-                                .build(),
+                                .build_dangling(),
                             lifetime: TestLifetime::Static,
                             is_dedicated: false,
                             is_optimal: false,
@@ -224,7 +222,7 @@ fn allocator_reverse_free() {
                                 .alignment(1024)
                                 .size(256)
                                 .memory_type_bits(u32::MAX)
-                                .build(),
+                                .build_dangling(),
                             lifetime: TestLifetime::Static,
                             is_dedicated: false,
                             is_optimal: false,
@@ -297,7 +295,7 @@ fn allocator_free_every_second_time() {
                                 .alignment(1024)
                                 .size(1024)
                                 .memory_type_bits(u32::MAX)
-                                .build(),
+                                .build_dangling(),
                             lifetime: TestLifetime::Static,
                             is_dedicated: false,
                             is_optimal: false,
@@ -358,7 +356,7 @@ fn allocator_allocation_dedicated() {
                         .alignment(512)
                         .size(10 * 1024 * 1024) // 10 MiB
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -404,7 +402,7 @@ fn allocator_properly_merge_free_entries() {
                         .alignment(256)
                         .size(256)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -420,7 +418,7 @@ fn allocator_properly_merge_free_entries() {
                         .alignment(256)
                         .size(256)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -436,7 +434,7 @@ fn allocator_properly_merge_free_entries() {
                         .alignment(256)
                         .size(256)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -452,7 +450,7 @@ fn allocator_properly_merge_free_entries() {
                         .alignment(256)
                         .size(256)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -481,13 +479,13 @@ fn allocator_fuzzy() {
         .unwrap();
 
         let mut allocations: Vec<(u8, Allocation<_>)> = Vec::with_capacity(10_0000);
-        let mut rng = Xoshiro256PlusPlus::seed_from_u64(0);
+        let rng = Rng::from_seed_with_64bit(42);
 
         for i in 0..10_000 {
-            let is_allocation = rng.gen_bool(0.6);
+            let is_allocation = rng.f32() <= 0.6;
 
             if (is_allocation && !allocations.is_empty()) || allocations.is_empty() {
-                let value: u8 = rng.gen_range(0..=255);
+                let value = rng.u8();
                 let size = (value as usize + 1) * 4;
 
                 let mut allocation = alloc
@@ -499,7 +497,7 @@ fn allocator_fuzzy() {
                                 .alignment(256)
                                 .size(size as u64)
                                 .memory_type_bits(u32::MAX)
-                                .build(),
+                                .build_dangling(),
                             lifetime: TestLifetime::Static,
                             is_dedicated: false,
                             is_optimal: false,
@@ -512,7 +510,7 @@ fn allocator_fuzzy() {
                 slice.fill(value);
                 allocations.push((value, allocation));
             } else {
-                let select: usize = rng.gen_range(0..allocations.len());
+                let select = rng.mod_usize(allocations.len());
                 let (value, allocation) = allocations.remove(select);
                 let slice = allocation.mapped_slice().unwrap().unwrap();
 
@@ -529,6 +527,14 @@ fn allocator_fuzzy() {
     }
 }
 
+fn align_down(val: u64, alignment: u64) -> u64 {
+    val & !(alignment - 1u64)
+}
+
+fn align_up(val: u64, alignment: u64) -> u64 {
+    align_down(val + alignment - 1u64, alignment)
+}
+
 #[test]
 fn allocator_granularity() {
     unsafe {
@@ -549,7 +555,7 @@ fn allocator_granularity() {
                         .alignment(256)
                         .size(512)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
@@ -560,6 +566,8 @@ fn allocator_granularity() {
         assert_eq!(allocation1.size(), 512);
         assert_eq!(allocation1.offset(), 0);
 
+        let optimal_align = align_up(allocation1.size(), ctx.buffer_image_granularity);
+
         let allocation2 = alloc
             .allocate(
                 &ctx.logical_device,
@@ -569,7 +577,7 @@ fn allocator_granularity() {
                         .alignment(256)
                         .size(1024)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: true,
@@ -578,7 +586,7 @@ fn allocator_granularity() {
             .unwrap();
 
         assert_eq!(allocation2.size(), 1024);
-        assert_eq!(allocation2.offset(), 1024);
+        assert_eq!(allocation2.offset(), optimal_align);
 
         alloc.deallocate(&ctx.logical_device, &allocation2).unwrap();
 
@@ -591,7 +599,7 @@ fn allocator_granularity() {
                         .alignment(256)
                         .size(1024)
                         .memory_type_bits(u32::MAX)
-                        .build(),
+                        .build_dangling(),
                     lifetime: TestLifetime::Static,
                     is_dedicated: false,
                     is_optimal: false,
